__all__ = ("get_git_version")

from subprocess import PIPE, Popen


def call_git_describe(abbrev):
    try:
        p = Popen(['git', 'describe', '--abbrev=%d' % abbrev],
                  stdout=PIPE, stderr=PIPE)
        p.stderr.close()
        line = p.stdout.readlines()[0]
        return line.strip().decode('utf-8')

    except:
        return None


def is_dirty():
    try:
        p = Popen(["git", "diff-index", "--name-only", "HEAD"],
                  stdout=PIPE, stderr=PIPE)
        p.stderr.close()
        lines = p.stdout.readlines()
        return len(lines) > 0
    except:
        return False


def read_release_version():
    try:
        f = open("RELEASE-VERSION", "r")

        try:
            version = f.readlines()[0]
            return version.strip()

        finally:
            f.close()

    except:
        return None


def write_release_version(version):
    f = open("RELEASE-VERSION", "w")
    f.write("%s\n" % version)
    f.close()


def get_git_version(abbrev=7):
    # Read in the version that's currently in RELEASE-VERSION.

    release_version = read_release_version()

    # First try to get the current version using “git describe”.

    version = call_git_describe(abbrev)
    if is_dirty():
        version += "-dirty"

    # If that doesn't work, fall back on the value that's in
    # RELEASE-VERSION.

    if version is None:
        version = release_version

    # If we still don't have anything, that's an error.

    if version is None:
        raise ValueError("Cannot find the version number!")

    # If the current version is different from what's in the
    # RELEASE-VERSION file, update the file to be current.

    if version != release_version:
        write_release_version(version)

    # Finally, return the current version.

    return version


if __name__ == "__main__":
    print(get_git_version())



from unittest import TestCase

from .context import extract_informations


class ExtractInformationsTest(TestCase):

    def test_should_have_tags(self):
        headers = {'params': {'tags': ['cat', 'dog']},
                   'operations': ['to_save'],
                   'pipeline_id': 'idp',
                   'step_id': 'ids',
                   'name': 'test',
                   'extension': 'jpeg'}

        params, operations, _, _, name, extension = extract_informations(headers)

        self.assertEqual(params, {'tags': ['cat', 'dog']})
        self.assertEqual(operations, ['to_save'])
        self.assertEqual(name, "test")
        self.assertEqual(extension, "jpeg")



import pika


def extract_informations(headers):
    params = headers['params'] if 'params' in headers else None
    operations = headers['operations'] if 'operations' in headers else None
    pipeline_id = headers['pipeline_id']
    step_id = headers['step_id']
    name = headers['name']
    extension = headers['extension']

    return params, operations, pipeline_id, step_id, name, extension


def resend(body, params, pipeline_id, step_id, consumer_id, status, reason, name, extension, amqp_url, amqp_port=5672):

    headers = {'pipeline_id': pipeline_id, 'step_id': step_id, 'consumer_id': consumer_id, 'status': status,
               'reason': reason, 'name': name, 'extension': extension, 'params': params}

    credentials = pika.PlainCredentials('guest', 'guest')
    parameters = pika.ConnectionParameters('10.0.0.8', amqp_port, '/', credentials)
    connection = pika.BlockingConnection(parameters)

    channel = connection.channel()

    channel.queue_declare(queue='job-finish', durable=True,
                          arguments={'x-dead-letter-exchange': '',
                                   'x-dead-letter-routing-key': 'job-finish.dead-letter.queue'})

    channel.basic_publish(exchange='',
                          routing_key='job-finish',
                          properties=pika.BasicProperties(
                              headers=headers
                          ),
                          body=body)

    connection.close()
    """with open(f"{name}.{extension}", "wb") as image:
        image.write(body)

    with open(f"{name}.{extension}", "rb") as image:
        files = {"media": image}
        content = {"pipeline_id": pipeline_id, "step_id": step_id, "consumer_id": consumer_id, "params": params}
        if reason:
            content['reason'] = reason

        data = {"data": [content]}
        # requests.post(url, files=files, data=data)"""



import json

import click
import coloredlogs
import pika

from ..consumer import ReconnectingExampleConsumer
from .main import load_consumer_file, find_consumer_file


@click.command(name='consumer')
@click.option('--file', '-f', default='consumer.py', required=True, help='Consumer file action')
@click.option('--log-level', '-l', default='INFO', help='Log level')
@click.option('--amqp-username', '-u', default='guest', help='AMQP Username')
@click.option('--amqp-password', '-p', default='guest', help='AMQP Password')
@click.option('--amqp-url', '-r', default='localhost', help='AMQP url')
@click.option('--amqp-port', '-P', default=5672, help='AMQP port')
@click.option('--queue-name', '-q', required=True, help='queue name to observe')
def cm(file, log_level, amqp_username, amqp_password, amqp_url, amqp_port, queue_name):
    coloredlogs.install(level=log_level)

    # Load implementation
    consumer_file = find_consumer_file(file)
    _,action_class = load_consumer_file(consumer_file)

    action = action_class[list(action_class.keys())[1]]()

    # Send registration message
    credentials = pika.PlainCredentials('guest', 'guest')
    parameters = pika.ConnectionParameters(amqp_url, amqp_port, '/', credentials)
    connection = pika.BlockingConnection(parameters)

    channel = connection.channel()
    channel.queue_declare(queue='registration',
                          durable=True,
                          arguments={'x-dead-letter-exchange': '',
                                     'x-dead-letter-routing-key': 'registration.dead-letter.queue'},
                          exclusive=False,
                          auto_delete=False)

    channel.confirm_delivery()

    channel.basic_publish(exchange='registration.exchange',
                          routing_key='registration.routing',
                          body=json.dumps({"name": action.name, "category": action.category}),
                          properties=pika.BasicProperties(
                              delivery_mode=2,  # make message persistent
                              content_type='text/plain'
                          ))

    amqp_url_joined = f'amqp://{amqp_username}:{amqp_password}@{amqp_url}:{amqp_port}/%2F'
    consumer = ReconnectingExampleConsumer(amqp_url_joined, queue_name, action)
    consumer.run()



import importlib
import inspect
import os
import sys

from logging_utils import LogDecorator

from consumers_core import Action


@LogDecorator('is-package-inspect')
def _is_package(path):
    return (
        os.path.isdir(path)
        and os.path.exists(os.path.join(path, '__init__.py'))
    )


@LogDecorator("find-consumer-file-inspect")
def find_consumer_file(consumer_file):
    names=[consumer_file]

    if not names[0].endswith('.py'):
        names.append(names[0] + '.py')

    if os.path.dirname(names[0]):
        for name in names:
            expanded = os.path.expanduser(name)
            if os.path.exists(expanded):
                if name.endswith('.py') or _is_package(expanded):
                    return os.path.abspath(expanded)
    else:
        path = os.path.abspath('.')
        while True:
            for name in names:
                joined = os.path.join(path, name)
                if os.path.exists(joined):
                    if name.endswith('.py') or _is_package(joined):
                        return os.path.abspath(joined)
            parent_path = os.path.dirname(path)
            if parent_path == path:
                break
            path = parent_path


@LogDecorator(decorator_log='is-action-class-inspect', resume=True)
def is_action_class(item):
    return bool(
        inspect.isclass(item)
        and issubclass(item, Action)
        and not isinstance(item, Action)
    )


@LogDecorator(decorator_log='load-consumer_file-inspect')
def load_consumer_file(path):

    @LogDecorator(decorator_log='import-consumer_file-inspect')
    def __import_consumer_file__(filename):
        spec = importlib.util.spec_from_file_location(os.path.splitext(filename)[0], path)
        action = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(action)
        return action

    sys.path.insert(0, os.getcwd())
    directory, consumer_file = os.path.split(path)

    print(f'directory: {directory}, filename: {consumer_file}')
    added_to_path = False
    index = None

    if directory not in sys.path:
        print(f"not in sys.path: {sys.path}")
        sys.path.insert(0, directory)
        added_to_path = True
    else:
        i = sys.path.index(directory)
        print(f"in sys.path {sys.path} index value: {i}")
        if i != 0:
            index = i
            sys.path.insert(0, directory)
            del sys.path[i + 1]

    imported = __import_consumer_file__(consumer_file)

    if added_to_path:
        del sys.path[0]

    if index is not None:
        sys.path.insert(index + 1, directory)
        del sys.path[0]

    consumer_classes = {name:value for name, value in vars(imported).items() if is_action_class(value)}
    return imported.__doc__, consumer_classes
